# -*- coding: utf-8 -*-

from reportlab.pdfgen import canvas
from reportlab.lib import colors
from reportlab.lib.units import inch
from reportlab.lib.styles import getSampleStyleSheet, ParagraphStyle
from reportlab.platypus import FrameBreak, PageBreak, Paragraph, Spacer
from reportlab.platypus import Frame, BaseDocTemplate, PageTemplate
from reportlab.lib.pagesizes import LETTER, landscape
from reportlab.platypus import Image, Table, TableStyle

class MyReport(BaseDocTemplate):

  class titlePageTemplate(PageTemplate):
    """
    Template for the first page of the document.
    """
    def __init__(self):
      content = Frame(0.5*inch, 0.5*inch, 10*inch, 6.5*inch)
      PageTemplate.__init__(self, "title_page", [content])

    def beforeDrawPage(self, canvas, doc):
      """
      Draw the title page
      """

      story = list()

      stylesheet=getSampleStyleSheet()

      title_style = stylesheet['Title']
      title = Paragraph("The Most Egregious Title",
                title_style)
      story.append(title)

      story.append(Spacer(0.5*inch, 0.25*inch))

      frame = self.frames[0]
      frame.addFromList(story, canvas)

    def afterDrawPage(self, canvas, doc):
      doc.handle_nextPageTemplate("graph_page")

  class graphPageTemplate(PageTemplate):
    """
    Template for each of the graph pages
    """
    def __init__(self):
      frames = list()

      ## Two frames for fixed paragraphs
      frames.append(Frame(5.1*inch, 6.8*inch,
                          5.5*inch, 0.8*inch,  showBoundary=1,
                          id="TopPara"))
      frames.append(Frame(5.1*inch, 5.8*inch,
                          5.5*inch, 0.8*inch,  showBoundary=1,
                          id="BotPara"))

      ## Frames for flowing graphs
      frames.append(Frame(0.4*inch, 2.5*inch,
                          4.5*inch, 4.5*inch, showBoundary=1,
                          id="Graph"))


      PageTemplate.__init__(self, "graph_page", frames)

    def beforeDrawPage(self, canvas, doc):
      self.drawParagraphs(canvas, doc)

    def afterDrawPage(self, canvas, doc):
      doc.handle_nextPageTemplate("table_page")

    def drawParagraphs(self, canvas, doc):
      style=getSampleStyleSheet()['Normal']
      p1 = Paragraph(
      """
      A very special paragraph that must go in every graph page. And
      it has to be long enough to fill up the region of the frame that
      has been alloted to it. There must absolutely not be any free
      space in this frame. ttttttttttttttttt asdf sd fsd f sasd fda sf dsa fx
      """, style)

      self.frames[0].add(p1, canvas)
      
      p2 = Paragraph(
      """
      A second special paragraph that must go in every graph page. And
      it has to be long enough to fill up the region of the frame that
      has been alloted to it. There must absolutely not be any free
      space in this frame. asd fds f adsfdsa fsd af dsa fasd fsda fds
      f af sd fsd fa dsfad sf df a asfafsdaf asdf dsa fa dsfs f asd
      fads f sdaf ads fa sdadsff ffffffffffffffff
      fffffffffffffffffffff ffffffffffffffff fffffffffff
      fffffffffffffffff ffffffffffffffffffffff
      """, style)
      self.frames[1].add(p2, canvas)



  class tablePageTemplate(PageTemplate):
    """
    Template for the pages with tables
    """
    def __init__(self):
      content = Frame(0.5*inch, 0.5*inch, 10*inch, 7*inch)
      PageTemplate.__init__(self, "table_page", [content])

    def afterDrawPage(self, canvas, doc):
      doc.handle_nextPageTemplate("graph_page")

  def __init__(self, filename, pagesize=[8.5*inch, 11*inch],
         conf="", months = []):
    BaseDocTemplate.__init__(self, filename, pagesize=pagesize)


  def handle_pageBegin(self):
    """
    Override base method to add a header and footer
    """
    self._handle_pageBegin()
    self.canv.setFontSize(12)
    self.canv.setFillColor(colors.black)
    self.canv.drawRightString( 10.5*inch, 8.2*inch, "[ohai – dere]" )
    self.canv.setFontSize(8)
    self.canv.setFillColor(colors.grey)
    self.canv.drawRightString( 10.5*inch, 0.3*inch,
                               "The Inexorable Page Footer" )

  def build(self, flowables):
    self._calc()
    self.addPageTemplates( [self.titlePageTemplate(),
                self.graphPageTemplate(),
                self.tablePageTemplate()])
    BaseDocTemplate.build(self, flowables, filename=self.filename)


def make_report():
  """
  This function actually puts everything together and generates the
  final report
  """

  story = list()
  doc = MyReport("report.pdf",
                 pagesize = landscape(LETTER))

  ## Start on second page, because first page is title page.
  story.append(PageBreak())

  ## Do three pairs of pages
  for k in range(0,3):

    ## Add an image
    im = Image("smiley.jpg", 4.45*inch, 3.80*inch)
    story.append(im)

    story.append(PageBreak())
    
    ## Add a table
    data = [[1, 2, 3, 4, 5],
            [6, 7, 8, 9, 10]]
    s = list()
    s.append( ('BOX', (0,0), (-1,-1), 1.5, colors.black))
    T = Table(data)
    T.setStyle(TableStyle(s))
    story.append(T)
    story.append(PageBreak())

  doc.build(story)


make_report()
